import os, json
from urllib.parse import urlparse
from pathlib import Path


def validate_domain(url_list):
    print("valid")


def get_all_definitions(path_to_json_files):
    json_file_names = [filename for filename in os.listdir(path_to_json_files) if filename.endswith('.json')]
    definitions = []
    for json_file_name in json_file_names:
        with open(os.path.join(path_to_json_files, json_file_name)) as json_file:
            try:
                json_content = json.load(json_file)
            except ValueError as err:
                raise Exception("Json schema not valid : " + json_file_name)
            definitions.append(json_content)

    return definitions


def validate_urls(url_list):
    parsed_url = urlparse(url_list[0])
    base_dom_name = parsed_url.netloc
    base_url = parsed_url.scheme + "://" + parsed_url.netloc + "/"
    for url in url_list:
        if base_dom_name != urlparse(url).netloc:
            print("error")
            return ""
    return base_url


def get_url_path(url):
    parsed_url = urlparse(url)
    return parsed_url.path


def merge_definition(ori_schemas, ori_paths, selected_definition, definition_path_prefix):
    selected_schemas = selected_definition["components"]["schemas"]
    selected_paths = selected_definition["paths"]
    selected_definition_title = selected_definition["info"]["title"].title().replace(" ", "")
    temp_schema = dict()
    for schema in selected_schemas:
        process_schema(ori_schemas, schema, selected_schemas, selected_paths, selected_definition_title, temp_schema)
    for schema in temp_schema.keys():
        ori_schemas[schema] = temp_schema[schema]
    for path in selected_definition["paths"].keys():
        ori_paths[definition_path_prefix + path] = selected_paths[path]


def process_schema(ori_schemas, selected_schema_name, selected_schemas, selected_paths, selected_definition_title,
                   temp_schema):
    rf_ext = check_ref_exists_in_schema(selected_schemas[selected_schema_name], selected_schema_name)
    # print(rf_ext)
    if not rf_ext:
        if selected_schema_name in ori_schemas:
            ori_schema = ori_schemas[selected_schema_name]
            selected_schema = selected_schemas[selected_schema_name]
            if ori_schema != selected_schema:
                print(
                    "schema not mactched " + selected_schema_name + " and insterted to original " +  selected_definition_title+selected_schema_name )
                ori_schemas[ selected_definition_title+selected_schema_name ] = selected_schemas[selected_schema_name]
                # del selected_schemas[schema]
                update_paths_by_ref(selected_paths, "#/components/schemas/" + selected_schema_name,
                                    "#/components/schemas/" +  selected_definition_title+selected_schema_name )
                update_schemas_by_ref(selected_schemas, "#/components/schemas/" + selected_schema_name,
                                      "#/components/schemas/" +  selected_definition_title+selected_schema_name )
                update_temp_schemas_by_ref(temp_schema, "#/components/schemas/" + selected_schema_name,
                                           "#/components/schemas/" +  selected_definition_title+selected_schema_name )
            else:
                print("schema  mactched  " + selected_schema_name)

        else:
            ori_schemas[selected_schema_name] = selected_schemas[selected_schema_name]
            print("not in original schema and inserted" + selected_schema_name)
    else:
        selected_schema = selected_schemas[selected_schema_name]
        if selected_schema_name in ori_schemas:
            ori_schm = ori_schemas[selected_schema_name]
            if ori_schm != selected_schema:
                print("schema not mactched  with ref " + selected_schema_name)
                temp_schema[ selected_definition_title+selected_schema_name ] = selected_schemas[selected_schema_name]
                # del selected_schemas[schema]
                update_paths_by_ref(selected_paths, "#/components/schemas/" + selected_schema_name,
                                    "#/components/schemas/" +  selected_definition_title+selected_schema_name )
                update_schemas_by_ref(selected_schemas, "#/components/schemas/" + selected_schema_name,
                                      "#/components/schemas/" +  selected_definition_title+selected_schema_name )
                update_temp_schemas_by_ref(temp_schema, "#/components/schemas/" + selected_schema_name,
                                           "#/components/schemas/" +  selected_definition_title+selected_schema_name )
        else:
            ori_schemas[selected_schema_name] = selected_schemas[selected_schema_name]

        print("have ref in schema ")


def get_schema_name_by_ref(ref):
    splited_list = ref.split("/")
    return splited_list[len(splited_list) - 1]


def check_ref_exists_in_schema(selected_schema, schema):
    if "properties" not in selected_schema:
        # print(schema)
        # print(selected_schema)
        return False
    else:
        properties = selected_schema["properties"]
        for schema_property in properties:
            if isinstance(properties[schema_property], dict):
                if "$ref" in properties[schema_property]:
                    return True
                elif "items" in properties[schema_property]:
                    if "$ref" in properties[schema_property]["items"]:
                        return True
                    else:
                        return False
                else:
                    print("selected schema not include ref or items ref")
                    return False
            else:
                print("selected schema is not dict")
                return False
    return schema


def update_paths_by_ref(selected_paths, current_ref, new_ref):
    for path in selected_paths:
        if "post" in selected_paths[path]:
            current_path = selected_paths[path]["post"]
            if "requestBody" in current_path or "responses" in current_path :
                if "requestBody" in current_path:
                    if current_ref == current_path["requestBody"]["content"]["application/json"]["schema"]["$ref"]:
                        selected_paths[path]["post"]["requestBody"]["content"]["application/json"]["schema"]["$ref"] = new_ref
                if "responses" in current_path:
                    if current_path == current_path["responses"]["200"]["content"]["application/json"]["schema"]["$ref"]:
                        selected_paths[path]["post"]["responses"]["200"]["content"]["application/json"]["schema"][
                            "$ref"] = new_ref
            else:
                print("No more requestBody or responses in : " + path)
        else:
            print("Incorrect path methode : " + selected_paths[path])


def update_schemas_by_ref(selected_schemas, current_ref, new_ref):
    for schema in selected_schemas:
        selected_schema = selected_schemas[schema]
        if "properties" in selected_schema:
            properties = selected_schema["properties"]
            for schema_property in properties:
                if isinstance(properties[schema_property], dict):
                    if "$ref" in properties[schema_property]:
                        if properties[schema_property]["$ref"] == current_ref:
                            selected_schemas[schema]["properties"][schema_property]["$ref"] = new_ref
                    elif "items" in properties[schema_property]:
                        if "$ref" in properties[schema_property]["items"]:
                            if properties[schema_property]["items"]["$ref"] == current_ref:
                                selected_schemas[schema]["properties"][schema_property]["items"]["$ref"] = new_ref
                    else:
                        print("No ref or items ref in " + schema_property)
                else:
                    print("Properties not a dict : " + properties[schema_property])
        else:
            print("Properties not include in " + schema)


def update_temp_schemas_by_ref(temp_schemas, current_ref, new_ref):
    for schema in temp_schemas:
        selected_schema = temp_schemas[schema]
        if "properties" in selected_schema:
            properties = selected_schema["properties"]
            for schema_property in properties:
                if isinstance(properties[schema_property], dict):
                    if "$ref" in properties[schema_property]:
                        if properties[schema_property]["$ref"] == current_ref:
                            temp_schemas[schema]["properties"][schema_property]["$ref"] = new_ref
                    elif "items" in properties[schema_property]:
                        if "$ref" in properties[schema_property]["items"]:
                            if properties[schema_property]["items"]["$ref"] == current_ref:
                                temp_schemas[schema]["properties"][schema_property]["items"]["$ref"] = new_ref
                    else:
                        print("No ref or items ref in " + schema_property)
                else:
                    print("Properties not a dict : " + properties[schema_property])
        else:
            print("Properties not include in " + schema)

    # for k in selected_schemas.copy():
    #     if get_schema_name_by_ref(current_ref) in k:
    #         selected_schemas[
    #             k.replace(get_schema_name_by_ref(current_ref), get_schema_name_by_ref(new_ref))] = selected_schemas.pop(
    #             k, 'e r r')
    # print(get_schema_name_by_ref(new_ref))
